// Compile on Unix:
// g++ -o Game Game.cpp GameEngine.cpp `sdl-config --libs` -lSDL_image -lSDL_ttf

#include <SDL/SDL.h>
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include <vector>
#include <sstream>
#include <string>
#include <iostream>
#include <fstream>
#include "GameEngine.h"
#include "Game.h"


/* ---------- StateMenu Methods ---------- */

StateMenu::StateMenu()
{
	Screen = GameEngine::load_image("menu.png");
}

StateMenu::~StateMenu()
{
	if (Screen != NULL) {
		SDL_FreeSurface(Screen);
		Screen = NULL;
	}
}

void StateMenu::Draw(SDL_Surface* surface)
{
	//Temporary rectangle to hold the offsets 
	SDL_Rect offset; 
	
	//Set the offsets 
	offset.x = 0; 
	offset.y = 0;
	
	//Blit the surface 
	SDL_BlitSurface( Screen, NULL, surface, &offset ); 
}

void StateMenu::KeyPressed(int key)
{
	if (key == 3) {
		GetEngine()->SetState(new StatePlay());
	} 
	else if (key == 4) {
		GetEngine()->SetState(new StateShowScore());
	}
}


/* ---------- StateShowScore Methods ---------- */

StateShowScore::StateShowScore()
{
	ReadFile();

	SDL_Surface* image;
	
	background = new Sprite();
	background->SetPos(0,0);
	image= GameEngine::load_image("showscorebg.png");
	background->SetImage(image);
	
	int posX = 50;
	int posY = 60;
	bool test = false;
	
	int size = filerows.size();
	if (size > 28) {
		size = 28;
	}
	for (int i=0; i < size; i++)
    	{
		Text* score = new Text();
		score->SetPos(posX,posY);
		score->Render(filerows[i].c_str());
		
		scores.push_back(score);
		
		posY += 30;

		if (i>12) {
			posX = 300;
			if (test == false) {
				posY = 60;
				test = true;
			}
		}
	}
}

StateShowScore::~StateShowScore()
{
	if (Screen != NULL) {
		SDL_FreeSurface(Screen);
		Screen = NULL;
	}
}

void StateShowScore::Draw(SDL_Surface* surface)
{
	background->Draw(surface);
	int size = scores.size();
    	for (int i=0; i < size; i++)
    	{
		scores[i]->Draw(surface);
	}
}

void StateShowScore::KeyPressed(int key)
{
	if (key == 3) {
		GetEngine()->SetState(new StateMenu());
	}
}

void StateShowScore::ReadFile()
{
	filename = "highscore.txt";
	std::string row;
	std::ifstream infile(filename.c_str());
	if (infile.is_open())
	{
		while (! infile.eof() )
 		{
			getline (infile,row);
			filerows.push_back(row);
		}
	}
	infile.close();
}


/* ---------- StateAddScore Methods ---------- */

StateAddScore::StateAddScore()
{
	SDL_Surface* image;
	
	background = new Sprite();
	background->SetPos(0,0);
	image= GameEngine::load_image("addscorebg.png");
	background->SetImage(image);
}

StateAddScore::~StateAddScore()
{
	delete background;
}

void StateAddScore::Draw(SDL_Surface* surface)
{
	background->Draw(surface);
}

void StateAddScore::KeyPressed(int key)
{
	if (key == 3) {
		ReadFile();
		AddScore();
		SortScore();
		WriteFile();
		GetEngine()->SetState(new StateShowScore());
	}
}

void StateAddScore::SetScore(int newscore)
{
	score = newscore;
}

void StateAddScore::ReadFile()
{
	GetEngine()->SetName();
	filename = "highscore.txt";
	std::string row;
	std::ifstream infile(filename.c_str());
	if (infile.is_open())
	{
		while (! infile.eof() )
 		{
			getline (infile,row);
			filerows.push_back(row);
		}
	}
	infile.close();
}

void StateAddScore::AddScore()
{
	std::string result;
	if (score < 10) {
		std::string temp = itos(score);
		result = "0";
		result.append( temp );
	} else { 
		result = itos(score);
	}
	std::string space = " points - ";
	result.append( space );
	result.append( GetName() );
	filerows.push_back(result);
}

void StateAddScore::SortScore()
{
	std::sort(filerows.begin(), filerows.end());
}

void StateAddScore::WriteFile()
{
	int size = filerows.size() - 1;

	std::ofstream outfile(filename.c_str());
	if (outfile.is_open()) {
    	for (int i=size; i >= 0; i--)
    	{
    		outfile << filerows[i] << std::endl;
    	}
  	}
}

std::string StateAddScore::itos(int i)	// convert int to string
{
	std::stringstream s;
	s << i;
	return s.str();
}


/* ---------- StateGameOver Methods ---------- */

StateGameOver::StateGameOver()
{
	SDL_Surface* image;
	
	background = new Sprite();
	background->SetPos(0,0);
	image= GameEngine::load_image("gameover.png");
	background->SetImage(image);
}

StateGameOver::~StateGameOver()
{
	delete background;
}

void StateGameOver::Draw(SDL_Surface* surface)
{
	background->Draw(surface);
}

void StateGameOver::KeyPressed(int key)
{
	if (key == 3) {
		StateAddScore* state = new StateAddScore();
		state->SetScore(score);
		GetEngine()->SetEntername();
		GetEngine()->SetNameentered();
		GetEngine()->SetState(state);
	}
}

void StateGameOver::SetScore(int newscore)
{
	score = newscore;
}


/* ---------- StatePlay Methods ---------- */

StatePlay::StatePlay()
{
	SDL_Surface* image;
	
	player = new PlayerSprite();
	player->SetPos(280,423);
	image= GameEngine::load_image("canon.png");
	player->SetImage(image);
	
	airplane = new AirplaneSprite();
	airplane->SetPos(-80,35);
	image= GameEngine::load_image("airplane.png");
	airplane->SetImage(image);
	
	background = new Sprite();
	background->SetPos(0,0);
	image= GameEngine::load_image("bg.png");
	background->SetImage(image);
	
	shield1 = new Sprite();
	shield1->SetPos(10,350);
	image= GameEngine::load_image("shield.png");
	shield1->SetImage(image);
	
	shield2 = new Sprite();
	shield2->SetPos(536,350);
	image= GameEngine::load_image("shield.png");
	shield2->SetImage(image);
	
	airplane2 = NULL;
	
	planesleft = 5;
	planesleft2 = 0;
	level = 1;
	score = 0;

	text_level = new Text();
	text_level->SetPos(290,11);
	std::string ss = itos(level);
	const char* p = ss.c_str();
	text_level->Render(p);

	text_score = new Text();
	text_score->SetPos(520,11);
	ss = itos(score);
	p = ss.c_str();
	text_score->Render(p);
}

StatePlay::~StatePlay()
{
	delete background;
	delete player;
	delete airplane;
	if (shield1 != NULL) {
		delete shield1;
	}
	if (shield2 != NULL) {
		delete shield2;
	}
	if (airplane2 != NULL) {
		delete airplane2;
	}
	int size = bullets.size();
    for (int i=0; i < size; i++)
    {
		delete bullets[i];
	}
	size = bombs.size();
    for (int i=0; i < size; i++)
    {
		delete bombs[i];
	}
}

void StatePlay::Draw(SDL_Surface* surface)
{
	background->Draw(surface); 
	player->Draw(surface);
	text_level->Draw(surface);
	text_score->Draw(surface);
	
	if (planesleft > 0) {
		airplane->Draw(surface);
	}
	if (planesleft2 > 0) {
		airplane2->Draw(surface);
	}
	
	int size = bullets.size();
    for (int i=0; i < size; i++)
    {
		bullets[i]->Draw(surface);
	}
	size = bombs.size();
    	for (int i=0; i < size; i++)
    	{
		bombs[i]->Draw(surface);
	}
	
	if (shield1 != NULL) {
		shield1->Draw(surface);
	}
	if (shield2 != NULL) {
		shield2->Draw(surface);
	}
}

void StatePlay::KeyPressed(int key)
{
	if (key == 4) {
		FireBullet(player->GetPosX(), player->GetPosY());
	}
	else if (key == 1) {
		player->MoveLeft();
	}
	else if (key == 2) {
		player->MoveRight();
	}
}

void StatePlay::MoveSprites()
{
	//move the plane(s) forward
	if (planesleft > 0) {
		airplane->MoveLeft();
	}
	if (planesleft2 > 0) {
		airplane2->MoveRight();
	}
	
	//if there's bullets
	int size = bullets.size();
    	for (int i=0; i < size; i++)
    	{
		if (bullets[i] != NULL) {
			
			bool collision1 = false;
			bool collision2 = false;
			bool collision3 = false;
			bool collision4 = false;
	    			    		
			//if the bullets are outside the window
	    	if (bullets[i]->GetPosY() < -10) {
	    		bullets[i]->SetDel();
			}

	    		//check for collisions
			if (airplane != NULL) {
				collision1 = bullets[i]->CollisionCheck(airplane->GetPosX(), airplane->GetPosY(), 73, 43);
	    		if (collision1 == true) {
	    			delete airplane;
	    			airplane = NULL;
					planesleft -= 1;
					score++;
					std::string ss = itos(score);
					const char* p = ss.c_str();
					text_score->Render(p);
	    		}
			}
	    	if (airplane2 != NULL) {
	    		collision2 = bullets[i]->CollisionCheck(airplane2->GetPosX(), airplane2->GetPosY(), 73, 43);
	    		if (collision2 == true) {
	    			delete airplane2;
	    			airplane2 = NULL;
					planesleft2 -= 1;
					score++;
					std::string ss = itos(score);
					const char* p = ss.c_str();
					text_score->Render(p);
	    		}
	    	}
	    	if (shield1 != NULL) {
	    		collision3 = bullets[i]->CollisionCheck(shield1->GetPosX(), shield1->GetPosY(), 95, 12);
	    	}
	    	if (shield2 != NULL) {
	    		collision4 = bullets[i]->CollisionCheck(shield2->GetPosX(), shield2->GetPosY(), 95, 12);
	    	}
	    	
	    	if (collision1 == false && collision2 == false && collision3 == false && collision4 == false) { 
	    		//move bullets
	    		bullets[i]->MoveUp();
			}
			if (collision1 == true || collision2 == true || collision3 == true || collision4 == true) {
				bullets[i]->SetDel();
			}
		}
		}
 
	//delete bullets outside windows or collsioned
	for (int i=size-1; i > 0; i--)
	{
		if (bullets[i]->GetDel() == true) {
			delete bullets[i];
			bullets.erase (bullets.begin()+i);
		}
	}

	//if the plane got shot
	if (airplane == NULL && planesleft > 0) {
		SDL_Surface* image;
		airplane = new AirplaneSprite();
		airplane->SetPos(-80,35);
		image= GameEngine::load_image("airplane.png");
		airplane->SetImage(image);
	}
	if (airplane2 == NULL && planesleft2 > 0) {
		SDL_Surface* image;
		airplane2 = new AirplaneSprite();
		airplane2->SetPos(-80,80);
		image= GameEngine::load_image("airplane2.png");
		airplane2->SetImage(image);
	}
	if (airplane == NULL && airplane2 == NULL && planesleft < 1 && planesleft2 < 1) {
		if (level < 5) {
			NextLevel();
		} else {
			StateGameOver* state = new StateGameOver;
			state->SetScore(score);
			GetEngine()->SetEntername();
			GetEngine()->SetState(state);
		}
	} else { //The airplanes drop bombs randomly
		if (airplane != NULL) {
			int shoot = rand() % (20 / level);
			if (shoot == 1) {
				DropBomb(airplane->GetPosX(), airplane->GetPosY());
			}
		}
		if (airplane2 != NULL) {
			int shoot = rand() % (20 / level);
			if (shoot == 1) {
				DropBomb(airplane2->GetPosX(), airplane2->GetPosY());
			}
		}
	}
	
	//move bombs
	size = bombs.size();
    	for (int i=0; i < size; i++)
    	{
		if (bombs[i] != NULL) {
	    	
			//if the bombs are outside the window
	    		if (bombs[i]->GetPosY() > 482) {
	    			bombs[i]->SetDel();
			}
			
			bool collision2 = false;
			bool collision3 = false;

	    	//check for collisions
			bool collision1 = bombs[i]->CollisionCheck(player->GetPosX(), player->GetPosY(), 72, 55);
			if (shield1 != NULL) {
				collision2 = bombs[i]->CollisionCheck(shield1->GetPosX(), shield1->GetPosY(), 95, 12);
			}
			if (shield2 != NULL) {
				collision3 = bombs[i]->CollisionCheck(shield2->GetPosX(), shield2->GetPosY(), 95, 12);
			}
	    	if (collision1 == true) {
				StateGameOver* state = new StateGameOver;
				state->SetScore(score);
				GetEngine()->SetEntername();
				GetEngine()->SetState(state);
	    	}
	    	else if (collision2 == true || collision3 == true) {
	    		bombs[i]->SetDel();
	    	}
	    	else { //move bombs
	    		bombs[i]->MoveDown();
	    	}
		}
    	}
	//delete bombs outside window or collisioned
	for (int i=size-1; i > 0; i--)
	{
		if (bombs[i]->GetDel()) {
			delete bombs[i];
			bombs.erase (bombs.begin()+i);
		}
	}
}

void StatePlay::FireBullet(int posX, int posY)
{
	SDL_Surface* image;
	
	BulletSprite* bullet = new BulletSprite();
	bullet->SetPos(posX+33,posY-3);
	image= GameEngine::load_image("bullet.png");
	bullet->SetImage(image);
	bullet->SetDel();
	
	bullets.push_back(bullet);
}

void StatePlay::DropBomb(int posX, int posY)
{
	SDL_Surface* image;
	
	BombSprite* bomb = new BombSprite();
	bomb->SetPos(posX+38,posY+35);
	image= GameEngine::load_image("bomb.png");
	bomb->SetImage(image);
	bomb->SetDel();
	
	bombs.push_back(bomb);
}

void StatePlay::NextLevel()
{
	level++;
	
	std::string ss = itos(level);
	const char* p = ss.c_str();
	text_level->Render(p);
	
	SDL_Surface* image;
		
	airplane = new AirplaneSprite();
	airplane->SetPos(-80,35);
	image= GameEngine::load_image("airplane.png");
	airplane->SetImage(image);
	
	airplane2 = new AirplaneSprite();
	airplane2->SetPos(750,80);
	image= GameEngine::load_image("airplane2.png");
	airplane2->SetImage(image);
	
	if (level == 2) {
		planesleft = 3;
		planesleft2 = 3;
		
		shield2 = NULL;
		shield1->SetPos(270,350);
	}
	else if (level == 3) {
		planesleft = 6;
		planesleft2 = 6;
		
		shield1->SetPos(536,350);
	}
	else if (level == 4) {
		planesleft = 100;
		planesleft2 = 100;
		
		shield1 = NULL;
	}
}

std::string StatePlay::itos(int i)	// convert int to string
{
	std::stringstream s;
	s << i;
	return s.str();
}


/* ---------- PlayerSprite Methods ---------- */

void PlayerSprite::MoveLeft()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	if (pos_x > 0) {
		int newposx = pos_x - 3;
		SetPos(newposx,pos_y);
	}
}

void PlayerSprite::MoveRight()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	if (pos_x < 594) {
		int newposx = pos_x + 3;
		SetPos(newposx,pos_y);
	}
}


/* ---------- AirplaneSprite Methods ---------- */

void AirplaneSprite::MoveLeft()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	if (pos_x < 650) {
		pos_x += 2;
		SetPos(pos_x,pos_y);
	} else {
		pos_x = -80;
		SetPos(pos_x,pos_y);
	}
}

void AirplaneSprite::MoveRight()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	if (pos_x > -80) {
		pos_x -= 4;
		SetPos(pos_x,pos_y);
	} else {
		pos_x = 650;
		SetPos(pos_x,pos_y);
	}
}


/* ---------- BulletSprite Methods ---------- */

void BulletSprite::MoveUp()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	pos_y -= 4;
	SetPos(pos_x,pos_y);
}

void BulletSprite::SetDel()
{
	if (del == false) {
		del = true;
	} else {
		del = false;
	}
}

bool BulletSprite::GetDel()
{
	return del;
}


/* ---------- BombSprite Methods ---------- */

void BombSprite::MoveDown()
{
	int pos_x = GetPosX();
	int pos_y = GetPosY();
	pos_y += 5;
	SetPos(pos_x,pos_y);
}

void BombSprite::SetDel()
{
	if (del == false) {
		del = true;
	} else {
		del = false;
	}
}

bool BombSprite::GetDel()
{
	return del;
}


/* ---------- Main Function ---------- */

int main()
{
	GameEngine* engine = new GameEngine();
	StateMenu* state = new StateMenu();
	engine->SetState(state);
	engine->Loop();
	
	delete engine;
	
	return 0;
}

