#ifndef GAME_H
#define GAME_H

#include <SDL/SDL.h>
#include <string>

/* ---------- Class Declarations for Sprites ---------- */

class PlayerSprite : public Sprite
{
	public:
		void MoveLeft();
		void MoveRight();
};

class AirplaneSprite : public Sprite
{
	public:
		void MoveLeft();
		void MoveRight();
};

class BulletSprite : public Sprite
{
	private:
		bool del;
	public:
		void MoveUp();
		void SetDel();
		bool GetDel();
};

class BombSprite : public Sprite
{
	private:
		bool del;
	public:
		void MoveDown();
		void SetDel();
		bool GetDel();
};


/* ---------- Class Declarations for Game States ---------- */

class StateMenu : public GameState
{
	private:
		SDL_Surface* Screen;
		GameEngine* engine;
	public:
		StateMenu();
		~StateMenu();
		void Draw(SDL_Surface* surface);
		void KeyPressed(int key);
};

class StateShowScore : public GameState
{
	private:
		SDL_Surface* Screen;
		GameEngine* engine;
		Sprite* background;
		std::string filename;
		std::vector<std::string> filerows;
		std::vector<Text*> scores;
	public:
		StateShowScore();
		~StateShowScore();
		void Draw(SDL_Surface* surface);
		void KeyPressed(int key);
		void ReadFile();
};

class StateAddScore : public GameState
{
	private:
		SDL_Surface* Screen;
		GameEngine* engine;
		Sprite* background;
		std::string filename;
		std::vector<std::string> filerows;
		int score;
	public:
		StateAddScore();
		~StateAddScore();
		void Draw(SDL_Surface* surface);
		void KeyPressed(int key);
		void SetScore(int newscore);
		void ReadFile();
		void AddScore();
		void SortScore();
		void WriteFile();
		std::string itos(int i);
};

class StateGameOver : public GameState
{
	private:
		SDL_Surface* Screen;
		GameEngine* engine;
		Sprite* background;
		int score;
	public:
		StateGameOver();
		~StateGameOver();
		void Draw(SDL_Surface* surface);
		void KeyPressed(int key);
		void SetScore(int newscore);
};

class StatePlay : public GameState
{
	private:
		SDL_Surface* Screen;
		GameEngine* engine;
		Sprite* background;
		PlayerSprite* player;
		AirplaneSprite* airplane;
		AirplaneSprite* airplane2;
		Sprite* shield1;
		Sprite* shield2;
		std::vector<BulletSprite*> bullets;
		std::vector<BombSprite*> bombs;
		int planesleft;
		int planesleft2;
		int level;
		int score;
		Text* text_level;
		Text* text_score;
	public:
		StatePlay();
		~StatePlay();
		void Draw(SDL_Surface* surface);
		void KeyPressed(int key);
		void MoveSprites();
		void FireBullet(int posX, int posY);
		void DropBomb(int posX, int posY);
		void NextLevel();
		std::string itos(int i);
};

#endif

